defmodule TelemetryUI.Slug do
  @separator_char ?-

  @moduledoc false

  @doc """
  Return a string in form of a slug for a given string.

  ## Examples

      iex> slugify(" Hi # there ")
      "Hi-there"

      iex> slugify("Über den Wölkchen draußen im Tore")
      "Ueber-den-Woelkchen-draussen-im-Tore"

      iex> slugify("Wikipedia Style", ?_)
      "Wikipedia_Style"

      iex> slugify("_Trimming_and___Removing_inside___")
      "Trimming-and-Removing-inside"

  """
  @spec slugify(text :: any, separator :: char) :: String.t()
  def slugify(text, separator \\ @separator_char) do
    text
    |> handle_possessives()
    |> replace_special_chars()
    |> String.downcase()
    |> remove_unwanted_chars(separator, ~r/([^A-Za-z0-9가-힣])+/)
  end

  defp remove_unwanted_chars(text, separator, pattern) do
    sep_binary = to_string([separator])

    text
    |> String.replace(pattern, sep_binary)
    |> String.trim(sep_binary)
  end

  defp replace_special_chars(text) do
    text
    |> to_charlist()
    |> replace_chars()
    |> to_string()
  end

  # Generate replacement functions using pattern matching.
  replacements = [
    # This file was generated from these two files:
    # https://github.com/easybook/slugger/blob/master/src/Easybook/Slugger.php
    # https://github.com/keyvanakbary/slugifier/blob/master/src/Slugifier.php
    # https://github.com/simov/slugify/blob/master/lib/slugify.js

    # Latin
    {?°, '0'},
    {?æ, 'ae'},
    {?ǽ, 'ae'},
    {?À, 'A'},
    {?Á, 'A'},
    {?Â, 'A'},
    {?Ã, 'A'},
    {?Å, 'AA'},
    {?Ǻ, 'A'},
    {?Ă, 'A'},
    {?Ǎ, 'A'},
    {?Æ, 'AE'},
    {?Ǽ, 'AE'},
    {?à, 'a'},
    {?á, 'a'},
    {?â, 'a'},
    {?ã, 'a'},
    {?å, 'aa'},
    {?ǻ, 'a'},
    {?ă, 'a'},
    {?ǎ, 'a'},
    {?ª, 'a'},
    {?@, 'at'},
    {?Ĉ, 'C'},
    {?Ċ, 'C'},
    {?ĉ, 'c'},
    {?ċ, 'c'},
    {?Ð, 'D'},
    {?Đ, 'D'},
    {?ó, 'o'},
    {?ð, 'd'},
    {?đ, 'd'},
    {?È, 'E'},
    {?É, 'E'},
    {?Ê, 'E'},
    {?Ë, 'E'},
    {?Ĕ, 'E'},
    {?Ė, 'E'},
    {?è, 'e'},
    {?é, 'e'},
    {?ê, 'e'},
    {?ë, 'e'},
    {?ĕ, 'e'},
    {?ė, 'e'},
    {?ƒ, 'f'},
    {?Ĝ, 'G'},
    {?Ġ, 'G'},
    {?ĝ, 'g'},
    {?ġ, 'g'},
    {?Ĥ, 'H'},
    {?Ħ, 'H'},
    {?ĥ, 'h'},
    {?ħ, 'h'},
    {?Ì, 'I'},
    {?Í, 'I'},
    {?Î, 'I'},
    {?Ï, 'I'},
    {?Ĩ, 'I'},
    {?Ĭ, 'I'},
    {?Ǐ, 'I'},
    {?Į, 'I'},
    {?Ĳ, 'IJ'},
    {?ì, 'i'},
    {?í, 'i'},
    {?î, 'i'},
    {?ï, 'i'},
    {?ĩ, 'i'},
    {?ĭ, 'i'},
    {?ǐ, 'i'},
    {?į, 'i'},
    {?ĳ, 'ij'},
    {?Ĵ, 'J'},
    {?ĵ, 'j'},
    {?Ĺ, 'L'},
    {?Ľ, 'L'},
    {?Ŀ, 'L'},
    {?ĺ, 'l'},
    {?ľ, 'l'},
    {?ŀ, 'l'},
    {?Ñ, 'N'},
    {?ñ, 'n'},
    {?ŉ, 'n'},
    {?Ò, 'O'},
    {?Ô, 'O'},
    {?Õ, 'O'},
    {?Ō, 'O'},
    {?Ŏ, 'O'},
    {?Ǒ, 'O'},
    {?Ő, 'O'},
    {?Ơ, 'O'},
    {?Ø, 'OE'},
    {?Ǿ, 'O'},
    {?Œ, 'OE'},
    {?ò, 'o'},
    {?ô, 'o'},
    {?õ, 'o'},
    {?ō, 'o'},
    {?ŏ, 'o'},
    {?ǒ, 'o'},
    {?ő, 'o'},
    {?ơ, 'o'},
    {?ø, 'oe'},
    {?ǿ, 'o'},
    {?º, 'o'},
    {?œ, 'oe'},
    {?Ŕ, 'R'},
    {?Ŗ, 'R'},
    {?ŕ, 'r'},
    {?ŗ, 'r'},
    {?Ŝ, 'S'},
    {?Ș, 'S'},
    {?ŝ, 's'},
    {?ș, 's'},
    {?ſ, 's'},
    {?Ţ, 'T'},
    {?Ț, 'T'},
    {?Ŧ, 'T'},
    {?Þ, 'TH'},
    {?ţ, 't'},
    {?ț, 't'},
    {?ŧ, 't'},
    {?þ, 'th'},
    {?Ù, 'U'},
    {?Ú, 'U'},
    {?Û, 'U'},
    {?Ũ, 'U'},
    {?Ŭ, 'U'},
    {?Ű, 'U'},
    {?Ų, 'U'},
    {?Ư, 'U'},
    {?Ǔ, 'U'},
    {?Ǖ, 'U'},
    {?Ǘ, 'U'},
    {?Ǚ, 'U'},
    {?Ǜ, 'U'},
    {?ù, 'u'},
    {?ú, 'u'},
    {?û, 'u'},
    {?ũ, 'u'},
    {?ŭ, 'u'},
    {?ű, 'u'},
    {?ų, 'u'},
    {?ư, 'u'},
    {?ǔ, 'u'},
    {?ǖ, 'u'},
    {?ǘ, 'u'},
    {?ǚ, 'u'},
    {?ǜ, 'u'},
    {?Ŵ, 'W'},
    {?ŵ, 'w'},
    {?Ý, 'Y'},
    {?Ÿ, 'Y'},
    {?Ŷ, 'Y'},
    {?ý, 'y'},
    {?ÿ, 'y'},
    {?ŷ, 'y'},
    {?Ç, 'C'},
    {?Ó, 'O'},
    {?ç, 'c'},

    # Arabic
    {?أ, 'a'},
    {?ب, 'b'},
    {?ت, 't'},
    {?ث, 'th'},
    {?ج, 'g'},
    {?ح, 'h'},
    {?خ, 'kh'},
    {?د, 'd'},
    {?ذ, 'th'},
    {?ر, 'r'},
    {?ز, 'z'},
    {?س, 's'},
    {?ش, 'sh'},
    {?ص, 's'},
    {?ض, 'd'},
    {?ط, 't'},
    {?ظ, 'th'},
    {?ع, 'aa'},
    {?غ, 'gh'},
    {?ف, 'f'},
    {?ق, 'k'},
    {?ك, 'k'},
    {?ل, 'l'},
    {?م, 'm'},
    {?ن, 'n'},
    {?ه, 'h'},
    {?و, 'o'},
    {?ي, 'y'},

    # Croatian
    # {?Đ, 'D'}, {?đ, 'd'},

    # Czech
    {?Č, 'C'},
    {?Ď, 'D'},
    {?Ě, 'E'},
    {?Ň, 'N'},
    {?Ř, 'R'},
    {?Š, 'S'},
    {?Ť, 'T'},
    {?Ů, 'U'},
    {?Ž, 'Z'},
    {?č, 'c'},
    {?ď, 'd'},
    {?ě, 'e'},
    {?ň, 'n'},
    {?ř, 'r'},
    {?š, 's'},
    {?ť, 't'},
    {?ů, 'u'},
    {?ž, 'z'},

    # Esperanto
    # {?Ĉ, 'C'}, {?ĉ, 'c'}, {?Ŝ, 'S'}, {?ŝ, 's'}, {?Ĥ, 'H'}, {?ĥ, 'h'},
    # {?Ŭ, 'U'}, {?ŭ, 'u'}, {?Ĵ, 'J'}, {?ĵ, 'j'}, {?Ĝ, 'G'}, {?ĝ, 'g'},

    # German
    {?Ä, 'Ae'},
    {?Ö, 'Oe'},
    {?Ü, 'Ue'},
    {?ß, 'ss'},
    {?ä, 'ae'},
    {?ö, 'oe'},
    {?ü, 'ue'},

    # Greek
    {?Α, 'A'},
    {?Β, 'B'},
    {?Γ, 'G'},
    {?Δ, 'D'},
    {?Ε, 'E'},
    {?Ζ, 'Z'},
    {?Η, 'H'},
    {?Θ, 'TH'},
    {?Ι, 'I'},
    {?Κ, 'K'},
    {?Λ, 'L'},
    {?Μ, 'M'},
    {?Ν, 'N'},
    {?Ξ, 'KS'},
    {?Ο, 'O'},
    {?Π, 'P'},
    {?Ρ, 'R'},
    {?Σ, 'S'},
    {?Τ, 'T'},
    {?Υ, 'Y'},
    {?Φ, 'F'},
    {?Χ, 'X'},
    {?Ψ, 'PS'},
    {?Ω, 'W'},
    {?Ϊ, 'I'},
    {?Ϋ, 'Y'},
    {?ά, 'a'},
    {?έ, 'e'},
    {?ή, 'h'},
    {?ί, 'i'},
    {?ΰ, 'y'},
    {?α, 'a'},
    {?β, 'b'},
    {?γ, 'g'},
    {?δ, 'd'},
    {?ε, 'e'},
    {?ζ, 'z'},
    {?η, 'h'},
    {?θ, 'th'},
    {?ι, 'i'},
    {?κ, 'k'},
    {?λ, 'l'},
    {?μ, 'm'},
    {?ν, 'n'},
    {?ξ, 'ks'},
    {?ο, 'o'},
    {?π, 'p'},
    {?ρ, 'r'},
    {?ς, 's'},
    {?σ, 's'},
    {?τ, 't'},
    {?υ, 'y'},
    {?φ, 'f'},
    {?χ, 'x'},
    {?ψ, 'ps'},
    {?ω, 'w'},
    {?ϊ, 'i'},
    {?ϋ, 'y'},
    {?ό, 'o'},
    {?ύ, 'y'},
    {?ώ, 'w'},
    {?ϒ, 'Y'},
    {?Ά, 'A'},
    {?Έ, 'E'},
    {?Ί, 'I'},
    {?Ό, 'O'},
    {?Ύ, 'Y'},
    {?Ή, 'H'},
    {?Ώ, 'W'},
    {?ΐ, 'i'},

    # Latvian
    {?Ā, 'A'},
    {?Ē, 'E'},
    {?Ģ, 'G'},
    {?Ī, 'I'},
    {?Ķ, 'K'},
    {?Ļ, 'L'},
    {?Ņ, 'N'},
    {?Ū, 'U'},
    {?ū, 'u'},
    # {?Ō, 'O'}, {?ō, 'o'}, {?Ŗ, 'R'}, {?ŗ, 'r'},
    {?ā, 'a'},
    {?ē, 'e'},
    {?ģ, 'g'},
    {?ī, 'i'},
    {?ķ, 'k'},
    {?ļ, 'l'},
    {?ņ, 'n'},

    # Lithuanian
    # {?Į, 'I'}, {?į, 'i'}, {?Ų, 'U'}, {?ų, 'u'}, {?Ė, 'E'}, {?ė, 'e'}, {?Ĩ, 'I'}, {?ĩ, 'i'},

    # Maltese
    # {?Ċ, 'C'}, {?ċ, 'c'}, {?Ġ, 'G'}, {?ġ, 'g'}, {?Ħ, 'H'}, {?ħ, 'h'},

    # Polish
    {?Ą, 'A'},
    {?Ć, 'C'},
    {?Ę, 'E'},
    {?Ł, 'L'},
    {?Ń, 'N'},
    {?Ś, 'S'},
    {?Ź, 'Z'},
    {?Ż, 'Z'},
    {?ą, 'a'},
    # {?Ó, 'O'}, {?ó, 'o'},
    {?ć, 'c'},
    {?ę, 'e'},
    {?ł, 'l'},
    {?ń, 'n'},
    {?ś, 's'},
    {?ź, 'z'},
    {?ż, 'z'},

    # Russian
    {?А, 'A'},
    {?Б, 'B'},
    {?В, 'V'},
    {?Г, 'G'},
    {?Д, 'D'},
    {?Е, 'E'},
    {?Ё, 'E'},
    {?Ж, 'Zh'},
    {?З, 'Z'},
    {?И, 'I'},
    {?Й, 'J'},
    {?К, 'K'},
    {?Л, 'L'},
    {?М, 'M'},
    {?Н, 'N'},
    {?О, 'O'},
    {?П, 'P'},
    {?Р, 'R'},
    {?С, 'S'},
    {?Т, 'T'},
    {?У, 'U'},
    {?Ф, 'F'},
    {?Х, 'H'},
    {?Ц, 'C'},
    {?Ч, 'Ch'},
    {?Ш, 'Sh'},
    {?Щ, 'Shch'},
    {?Ъ, ''},
    {?Ы, 'Y'},
    {?Ь, ''},
    {?Э, 'E'},
    {?Ю, 'Ju'},
    {?Я, 'Ja'},
    {?а, 'a'},
    {?б, 'b'},
    {?в, 'v'},
    {?г, 'g'},
    {?д, 'd'},
    {?е, 'e'},
    {?ё, 'e'},
    {?ж, 'zh'},
    {?з, 'z'},
    {?и, 'i'},
    {?й, 'j'},
    {?к, 'k'},
    {?л, 'l'},
    {?м, 'm'},
    {?н, 'n'},
    {?о, 'o'},
    {?п, 'p'},
    {?р, 'r'},
    {?с, 's'},
    {?т, 't'},
    {?у, 'u'},
    {?ф, 'f'},
    {?х, 'h'},
    {?ц, 'c'},
    {?ч, 'ch'},
    {?ш, 'sh'},
    {?щ, 'shch'},
    {?ъ, ''},
    {?ы, 'y'},
    {?ь, ''},
    {?э, 'e'},
    {?ю, 'ju'},
    {?я, 'ja'},

    # Sami
    {?Ŋ, 'N'},
    {?ŋ, 'n'},
    {?Ǧ, 'G'},
    {?ǧ, 'g'},
    {?Ǩ, 'K'},
    {?ǩ, 'k'},
    {?Ʒ, 'Z'},
    {?ʒ, 'z'},
    {?Ǯ, 'Z'},
    {?ǯ, 'z'},

    # Serbian
    {?ђ, 'dj'},
    {?ј, 'j'},
    {?љ, 'lj'},
    {?њ, 'nj'},
    {?ћ, 'c'},
    {?џ, 'dz'},
    {?Ђ, 'Dj'},
    {?Ј, 'j'},
    {?Љ, 'Lj'},
    {?Њ, 'Nj'},
    {?Ћ, 'C'},
    {?Џ, 'Dz'},

    # Slovak
    # {?Ĺ, 'L'}, {?ĺ, 'l'}, {?Ŕ, 'R'}, {?ŕ, 'r'}, {?Ľ, 'L'}, {?ľ, 'l'}

    # Turkish
    # {?Ç, 'C'}, {?ç, 'c'},
    {?Ğ, 'G'},
    {?İ, 'I'},
    {?Ş, 'S'},
    {?ğ, 'g'},
    {?ı, 'i'},
    {?ş, 's'},

    # Ukrainian
    {?Є, 'Ye'},
    {?І, 'I'},
    {?Ї, 'Ji'},
    {?Ґ, 'G'},
    {?є, 'ye'},
    {?і, 'i'},
    {?ї, 'ji'},
    {?ґ, 'g'},

    # Vietnamese
    {?ạ, 'a'},
    {?ả, 'a'},
    {?ầ, 'a'},
    {?ấ, 'a'},
    {?ậ, 'a'},
    {?ẩ, 'a'},
    {?ẫ, 'a'},
    {?ằ, 'a'},
    {?ắ, 'a'},
    {?ặ, 'a'},
    {?ẳ, 'a'},
    {?ẵ, 'a'},
    {?ẹ, 'e'},
    {?ẻ, 'e'},
    {?ẽ, 'e'},
    {?ề, 'e'},
    {?ế, 'e'},
    {?ệ, 'e'},
    {?ể, 'e'},
    {?ễ, 'e'},
    {?ị, 'i'},
    {?ỉ, 'i'},
    {?ọ, 'o'},
    {?ỏ, 'o'},
    {?ồ, 'o'},
    {?ố, 'o'},
    {?ộ, 'o'},
    {?ổ, 'o'},
    {?ỗ, 'o'},
    {?ờ, 'o'},
    {?ớ, 'o'},
    {?ợ, 'o'},
    {?ở, 'o'},
    {?ỡ, 'o'},
    {?ụ, 'u'},
    {?ủ, 'u'},
    {?ừ, 'u'},
    {?ứ, 'u'},
    {?ự, 'u'},
    {?ử, 'u'},
    {?ữ, 'u'},
    {?ỳ, 'y'},
    {?ỵ, 'y'},
    {?ỷ, 'y'},
    {?ỹ, 'y'},
    {?Ạ, 'A'},
    {?Ả, 'A'},
    {?Ầ, 'A'},
    {?Ấ, 'A'},
    {?Ậ, 'A'},
    {?Ẩ, 'A'},
    {?Ẫ, 'A'},
    {?Ằ, 'A'},
    {?Ắ, 'A'},
    {?Ặ, 'A'},
    {?Ẳ, 'A'},
    {?Ẵ, 'A'},
    {?Ẹ, 'E'},
    {?Ẻ, 'E'},
    {?Ẽ, 'E'},
    {?Ề, 'E'},
    {?Ế, 'E'},
    {?Ệ, 'E'},
    {?Ể, 'E'},
    {?Ễ, 'E'},
    {?Ị, 'I'},
    {?Ỉ, 'I'},
    {?Ọ, 'O'},
    {?Ỏ, 'O'},
    {?Ồ, 'O'},
    {?Ố, 'O'},
    {?Ộ, 'O'},
    {?Ổ, 'O'},
    {?Ỗ, 'O'},
    {?Ờ, 'O'},
    {?Ớ, 'O'},
    {?Ợ, 'O'},
    {?Ở, 'O'},
    {?Ỡ, 'O'},
    {?Ụ, 'U'},
    {?Ủ, 'U'},
    {?Ừ, 'U'},
    {?Ứ, 'U'},
    {?Ự, 'U'},
    {?Ử, 'U'},
    {?Ữ, 'U'},
    {?Ỳ, 'Y'},
    {?Ỵ, 'Y'},
    {?Ỷ, 'Y'},
    {?Ỹ, 'Y'},

    # Other
    {?¹, '1'},
    {?², '2'},
    {?³, '3'},
    {?¶, 'P'},

    # Currency
    {?€, 'euro'},
    {?₢, 'cruzeiro'},
    {?₣, 'frenchfranc'},
    {?£, 'pound'},
    {?₤, 'lira'},
    {?₥, 'mill'},
    {?₦, 'naira'},
    {?₧, 'peseta'},
    {?₨, 'rupee'},
    {?₩, 'won'},
    {?₪, 'newshequel'},
    {?₫, 'dong'},
    {?₭, 'kip'},
    {?₮, 'tugrik'},
    {?₯, 'drachma'},
    {?₰, 'penny'},
    {?₱, 'peso'},
    {?₲, 'guarani'},
    {?₳, 'austral'},
    {?₴, 'hryvnia'},
    {?₵, 'cedi'},
    {?¢, 'cent'},
    {?¥, 'yen'},
    {?元, 'yuan'},
    {?円, 'yen'},
    {?﷼, 'rial'},
    {?₠, 'ecu'},
    {?¤, 'currency'},
    {?฿, 'baht'},
    {?$, 'dollar'},
    {?Ƀ, 'bitcoin'},
    {?₹, 'indianrupee'},

    # Symbols
    {?©, 'copyright'},
    {?∑, 'sum'},
    {?®, 'registered'},
    {?†, '+'},
    {?“, '"'},
    {?”, '"'},
    {?‘, '\''},
    {?’, '\''},
    {?∂, 'd'},
    {?™, 'tm'},
    {?℠, 'sm'},
    {?…, '...'},
    {?˚, 'o'},
    {?•, '*'},
    {?∆, 'delta'},
    {?∞, 'infinity'},
    {?♥, 'love'},
    {?&, 'and'},
    {?|, 'or'},
    {?<, 'less'},
    {?>, 'greater'},
    {?☢, 'radioactive'}
  ]

  replacements_by_search =
    replacements
    |> Enum.group_by(fn {search, _} -> search end, fn {_, replace} -> replace end)
    |> Enum.into([])

  # Create replacement
  for {search, replaces} <- replacements_by_search do
    if search != @separator_char do
      replace = hd(replaces)
      defp replace_chars([unquote(search) | t]), do: unquote(replace) ++ replace_chars(t)
    end
  end

  # A unmatched char will be kept.
  defp replace_chars([h | t]), do: [h] ++ replace_chars(t)

  # String has come to an end, stop recursion here.
  defp replace_chars([]), do: []

  # Handle "Sheep's Milk" so it will be "sheeps-milk" instead of "sheep-s-milk"
  defp handle_possessives(text) do
    String.replace(text, ~r/['’]s/u, "s")
  end
end
